"""Topology utilities for sagellm-comm."""

from __future__ import annotations

from sagellm_comm.topology.device import DeviceId
from sagellm_comm.topology.link import Link, LinkKind

__all__ = [
    "DeviceId",
    "Link",
    "LinkKind",
]
